
;; Representation: a List is a struct where kind is one of
;;   <'EMPTY ? ?>
;;   <'SINGLETON a ?>
;;   <'MERGE L M>

(define-struct list (kind field1 field2) #:transparent)

;; Creators

(define (L:empty) 
  (make-list 'EMPTY 0 0))

(define (L:singleton a) 
  (make-list 'SINGLETON a 0))

(define (L:merge L M) 
  (make-list 'MERGE L M))


;; Helper functions
(define (kind-empty? L)
  (eq? (list-kind L) 'EMPTY))
(define (kind-singleton? L)
  (eq? (list-kind L) 'SINGLETON))
(define (kind-merge? L)
  (eq? (list-kind L) 'MERGE))


;; Operations

(define (L:isEmpty L)
  (or (kind-empty? L)
      (and (kind-merge? L)
	   (L:isEmpty (list-field1 L))
	   (L:isEmpty (list-field2 L)))))

(define (L:first L)
  (cond [(kind-empty? L) (error "(L:first L): L is empty")]
	[(kind-singleton? L) (list-field1 L)]
	[(L:isEmpty (list-field1 L)) (L:first (list-field2 L))]
	[else (L:first (list-field1 L))]))

(define (L:rest L)
  (cond [(kind-empty? L) (error "(L:rest L): L is empty")]
	[(kind-singleton? L) (L:empty)]
	[(L:isEmpty (list-field1 L)) (L:rest (list-field2 L))]
	[else (L:merge (L:rest (list-field1 L)) (list-field2 L))]))

(define (L:length L)
  (cond [(kind-empty? L) 0]
	[(kind-singleton? L) 1]
	[else (+ (L:length (list-field1 L)) (L:length (list-field2 L)))]))

(define (L:isEqual L M)
  (or (and (L:isEmpty L) (L:isEmpty M))
      (and (not (L:isEmpty L)) (not (L:isEmpty M))
	   (= (L:first L) (L:first M))
	   (L:isEqual (L:rest L) (L:rest M)))))
