
(define-syntax (check-equal syntax-object)
  (syntax-case syntax-object ()
    ((_ expr result)
     #'(if (eq? expr result)
	   #t
	   (error "Test failed -- " (quote (check-equal expr result))
                  " -- expected " result " got " expr)))))

;; relies on correctness of other operations
(define (eqlist L M)
  (or (and (L:isEmpty L) (L:isEmpty M))
      (and (not (L:isEmpty L)) (not (L:isEmpty M))
	   (= (L:first L) (L:first M))
	   (eqlist (L:rest L) (L:rest M)))))

(define-syntax (check-equal-list syntax-object)
  (syntax-case syntax-object ()
    ((_ expr result)
     #'(if (eqlist expr result)
	   #t
	   (error "Test failed -- " (quote (check-equal-list expr result))
                  " -- expected " result " got " expr)))))
	   


(define (random-list)
  (define (list-size-n n)
    (cond ((= n 0) (L:empty))
	  ((= (modulo n 2) 0) (L:merge (list-size-n (/ n 2))
			               (list-size-n (/ n 2))))
	  (else (L:merge (L:singleton (random 30))
			 (L:merge (list-size-n (/ (- n 1) 2))
				  (list-size-n (/ (- n 1) 2)))))))
  (list-size-n (random 30)))

	 
(let ((L (random-list))
      (M (random-list))
      (n (random 30)))
  (check-equal (L:isEmpty (L:empty)) #t)
  (check-equal (L:isEmpty (L:singleton n)) #f)
  (check-equal (L:isEmpty (L:merge L M)) 
	       (if (and (L:isEmpty L) (L:isEmpty M)) #t #f))
  (check-equal (L:first (L:singleton n)) n)
  (check-equal (L:first (L:merge L M)) 
	       (if (L:isEmpty L) (L:first M) (L:first L)))
  (check-equal-list (L:rest (L:singleton n)) (L:empty))
  (check-equal-list (L:rest (L:merge L M))
		    (if (L:isEmpty L) (L:rest M) (L:merge (L:rest L) M)))
  (check-equal (L:length (L:empty)) 0)
  (check-equal (L:length (L:singleton n)) 1)
  (check-equal (L:length (L:merge L M)) (+ (L:length L) (L:length M)))
  ;; ... need to check for isEqual()
  (printf "All tests passed\n")
  )

