
(define-struct point (xpos ypos))

;; Creator (cartesian x-position y-position)
(define (cartesian x y) 
  (make-point x y))

;; Creator (polar distance-from-origin angle-with-positive-x-axis)
;;    note that we convert from polar to cartesian immediately
(define (polar r theta)
  (if (< r 0)
      (error "(polar r theta): r must be >= 0")
      (cartesian (* r (cos theta))
		 (* r (sin theta)))))



;; Helper functions (for profiling)

(define *count* 0)
(define (count-reset!) (set! *count* 0))
(define (count-incr!) (set! *count* (+ *count* 1)))

(define (xpos p)
  (count-incr!)
  (point-xpos p))

(define (ypos p)
  (count-incr!)
  (point-ypos p))



;; Selectors/Operations

(define (xCoord p) 
  (xpos p))

(define (yCoord p) 
  (ypos p))

(define (angle-with-x-axis p)
  (atan (yCoord p) (xCoord p)))

(define (distance-from-origin p)
  (distance p (cartesian 0 0)))

(define (distance p q) 
  (sqrt (+ (expt (- (xCoord p) (xCoord q)) 2)
	   (expt (- (yCoord p) (yCoord q)) 2))))

(define (move p dx dy) 
  (cartesian (+ (xCoord p) dx) 
	     (+ (yCoord p) dy)))

(define (add p q) 
  (cartesian (+ (xCoord p) (xCoord q))
	     (+ (yCoord p) (yCoord q))))

(define (rotate p theta) 
  (let ((x (xCoord p))
	(y (yCoord p)))
    (cartesian (- (* (cos theta) x) (* (sin theta) y))
	       (+ (* (sin theta) x) (* (cos theta) y)))))



;; Predicates

(define (isEqual p q) 
  (and (= (xCoord p) (xCoord q))
       (= (yCoord p) (yCoord q))))

(define (isOrigin p) 
  (and (= (xCoord p) 0) 
       (= (yCoord p) 0)))

