
(define-struct point (kind first second))

;; Creator (cartesian x-position y-position)
(define (cartesian x y) 
  (make-point 'CARTESIAN x y))

;; Creator (polar distance-from-origin angle-with-positive-x-axis)
;;    note that we convert from polar to cartesian immediately
(define (polar r theta)
  (if (< r 0)
      (error "(polar r theta): r must be >= 0")
      (make-point 'POLAR r theta)))


;; Some helper functions
(define *count* 0)
(define (count-reset!) (set! *count* 0))
(define (count-incr!) (set! *count* (+ *count* 1)))

(define (polar-rep? p)
  (eq? (point-kind p) 'POLAR))

(define (cartesian-rep? p)
  (eq? (point-kind p) 'CARTESIAN))

(define (fst p) 
  (count-incr!)
  (point-first p))

(define (snd p) 
  (count-incr!)
  (point-second p))


;; Selectors/Operations

(define (xCoord p) 
  (if (cartesian-rep? p)
      (fst p)
      (* (fst p) (cos (snd p)))))

(define (yCoord p) 
  (if (cartesian-rep? p)
      (snd p)
      (* (fst p) (sin (snd p)))))

(define (angle-with-x-axis p)
  (if (cartesian-rep? p)
      (atan (snd p) (fst p))
      (snd p)))

(define (distance-from-origin p)
  (if (cartesian-rep? p)
      (distance p (cartesian 0 0))
      (fst p)))
  
(define (distance p q) 
  (sqrt (+ (expt (- (xCoord p) (xCoord q)) 2)
	   (expt (- (yCoord p) (yCoord q)) 2))))

(define (move p dx dy) 
  (cartesian (+ (xCoord p) dx) 
	     (+ (yCoord p) dy)))

(define (add p q) 
  (cartesian (+ (xCoord p) (xCoord q))
	     (+ (yCoord p) (yCoord q))))

(define (rotate p theta) 
  (if (cartesian-rep? p)
      (cartesian (- (* (cos theta) (fst p)) (* (sin theta) (snd p)))
		 (+ (* (sin theta) (fst p)) (* (cos theta) (snd p))))
      (polar (fst p) (+ (snd p) theta))))


;; Predicates

(define (normalize angle)
  (cond ((>= angle (* 2 pi)) (normalize (- angle (* 2 pi))))
	((< angle 0) (normalize (+ angle (* 2 pi))))
	(else angle)))


;; BUGGY! Exercise -- spot the bug!
(define (isEqual p q) 
  (if (cartesian-rep? p)
      (and (= (fst p) (fst q))
	   (= (snd p) (snd q)))
      (and (= (fst p) (fst q))
	   (= (normalize (snd p)) (normalize (snd q))))))

(define (isOrigin p) 
  (if (cartesian-rep? p)
      (and (= (fst p) 0) 
	   (= (snd p) 0))
      (= (fst p) 0)))



